import numpy as np
import matplotlib.pyplot as plt
from scipy import interpolate
from mpl_toolkits.mplot3d import Axes3D
from sklearn.cluster import DBSCAN
from sklearn.neighbors import NearestNeighbors
import sys
import os

def check_file(filename):
   if not os.path.isfile(filename):
      return False
   else: return True

def read_file(filename):
	if not check_file(filename):
		sys.stderr.write('[ERROR] input file absent\n')
		return False
	data=[]
	with open(filename) as f:
		for i, line in enumerate(f, 1):
			line=line.rstrip()
			if line=="":continue
			if line[0]=="#":continue
			v=line.split()
			v=np.array([float(x) for x in line.split()])
			data.append(v)
	f.close()
	return np.array(data)

def cluster_analysis(infile):
	data = read_file(infile)
	if(len(data)==0):
		sys.stderr.write('[ERROR] not enough data to perfomr cluster analysis\n')
		return False


	"""
	fig = plt.figure()
	ax = Axes3D(fig)
	ax.scatter(data[:,0], data[:,1], data[:,2], s=10)
	ax.view_init(azim=200)
	plt.show()
	plt.savefig('initial_snap.png')
	"""


	#https://machinelearningknowledge.ai/tutorial-for-dbscan-clustering-in-python-sklearn/
	#https://towardsdatascience.com/explaining-dbscan-clustering-18eaf5c83b31
	#The optimal value for epsilon will be found at the point of maximum curvature.
	neighbors = NearestNeighbors(n_neighbors=20)
	neighbors_fit = neighbors.fit(data)
	distances, indices = neighbors_fit.kneighbors(data)

	distances = np.sort(distances, axis=0)
	distances = distances[:,1]
	MAX = np.amax(distances, axis=0)
	MIN = np.amin(distances, axis=0)
	sys.stderr.write('MAX_DIST = {:{w}.{p}f}\n'.format(MAX, w=15, p=15 ) )
	sys.stderr.write('MIN_DIST = {:{w}.{p}f}\n'.format(MIN, w=15, p=15 ) )
	plt.xlabel("Points")
	plt.ylabel("Distance")
	#plt.axis([0, 100, -0.001, 0.001])
	#plt.plot(distances)
	#plt.show()
	#plt.savefig('epsilon_snap.png')

	x = np.linspace(0, len(distances), num=len(distances))
	t,c,k = interpolate.splrep(x, distances, s=1)
	yfit = interpolate.BSpline(t,c,k)(x) 
	#z = np.polyfit(x, distances, 2)
	#ypre = z[0]*x*x  + x*z[1] + z[2]
	plt.plot(x, yfit)
	plt.show()
	plt.savefig('epsilon_snap.png')

	model = DBSCAN(eps=1.10, min_samples=5)
	model.fit_predict(data)
	pred = model.fit_predict(data)

	"""
	labels = model.labels_
	#Number of clusters in labels, ignoring noise if present.
	n_clusters_ = len(set(labels)) - (1 if -1 in labels else 0)
	n_noise_ = list(labels).count(-1)
	print('Estimated number of clusters: %d' % n_clusters_)
	print('Estimated number of noise points: %d' % n_noise_)
	"""

	unique_cluster_labels=[]
	unique_cluster_labels.append(-2)
	#print(model.labels_[0])
	#return
	for i in range(len(model.labels_)):
		state = True
		for j in range(len(unique_cluster_labels)):
			if model.labels_[i] == unique_cluster_labels[j]:
				state = False
				break
		if state is True:
			#print(model.labels_[i])
			unique_cluster_labels.append(model.labels_[i])
			

	unique_cluster_label_count = np.zeros((len(unique_cluster_labels)))
	for i in range(len(model.labels_)):
		for j in range(len(unique_cluster_labels)):
			if unique_cluster_labels[j] == model.labels_[i]:
				unique_cluster_label_count[j] = unique_cluster_label_count[j] + 1
				break

	#print(len(model.core_sample_indices_))
	#print(len(data))
	#print(model.n_features_in_)
	#print(model.components_[0])
	out = open('cluster_distribution.dat', 'w')
	for i in range(len(unique_cluster_labels)):
		out.write('{:{w}.{p}f}'.format(unique_cluster_labels[i], w=15, p=0 ) )
		out.write('{:{w}.{p}f}'.format(unique_cluster_label_count[i], w=15, p=0 ) )
		out.write('\n')
	out.close()

	#for i in range(len(unique_cluster_labels)):
	#	print(unique_cluster_labels[i], "  ", unique_cluster_label_count[i])



	fig = plt.figure()
	ax = Axes3D(fig)
	ax.scatter(data[:,0], data[:,1], data[:,2], c=model.labels_, s=20)
	ax.view_init(azim=000)
	ax.view_init(-140, 90)
	ax.view_init(azim=0, elev=90)
	plt.show()
	plt.savefig('Li-cluster.png')

	#db1_labels = model.labels_
	#labels, counts = np.unique(db1_labels[db1_labels>=0], return_counts=True)
	#print(labels[np.argsort(-counts)[:10]])

	#print("number of cluster found: {}".format(len(set(model.labels_))))
	#print('cluster for each point: ', model.labels_)
	#print(set(model.labels_))
