import os
import sys
import shutil
import numpy as np
import lica.ana_std as ana
import MDAnalysis as mda
import MDAnalysis.analysis.msd as msd
import MDAnalysis.analysis.diffusionmap as diffusionmap
from MDAnalysis.core.groups import AtomGroup
import LICA
import lica.atommaN as ATOMMAN


class gmax_analysis:
	def __init__(self):
		self.start = 0
		self.stop = 0
		self.step = 0
		self.Dir = ""
		self.outdir = ""
		self.elements = []
	def get_steps(self, start, stop, step):
		self.start = int(start)
		self.stop = int(stop)
		self.step = int(step)
	def get_dir_names(self, Dir, outdir):
		self.Dir = Dir
		self.outdir = outdir
	def get_elemenrs(self, elements):
		self.elements = elements
	def ananlysis(self):
		topology=self.Dir+'/nvt.tpr'
		if  ana.IsFileExist(topology) is False:
			sys.stderr.write('[ERROR] unable to read file: '+topology+'\n')
			return False, []
		binfile=self.Dir+'/nvt.trr'
		if ana.IsFileExist(binfile) is False:
			sys.stderr.write('[ERROR] unable to read file: '+ binfile+'\n')
			return False, []
		u = mda.Universe(topology,binfile)
		u.transfer_to_memory(start=self.start, stop=self.stop, step=self.step, verbose=False)

		names = 'name'
		for el_i in self.elements:
			names = names + ' ' +el_i

		ag = u.select_atoms(names, sorted=False, updating=True)

		filename = ""

		for ts in u.trajectory:
			filename = self.outdir + "/INFO_"+str(int(ts.time+self.start))
			out = open(filename, 'w')
			out.write('{:{w}.{p}f}'.format(ts.time+self.start, w=15, p=3 ) )
			out.write('{:{w}.{p}f}'.format(ts.dt, w=15, p=3 ) )
			out.write('\n')
			for i in range(3):
				for j in range(3):
					out.write('{:{w}.{p}f}'.format(ts.triclinic_dimensions[i][j], w=25, p=15 ) )
				out.write('\n')
			for i in range(len(ag.names)):
				out.write('{:>5}'.format(ag.names[i]))
				out.write('{:{w}.{p}f}'.format(ag.ix[i], w=10, p=0 ) )
				out.write('{:{w}.{p}f}'.format(ag.positions[i][0], w=15, p=8 ) )
				out.write('{:{w}.{p}f}'.format(ag.positions[i][1], w=15, p=8 ) )
				out.write('{:{w}.{p}f}'.format(ag.positions[i][2], w=15, p=8 ) )
				if ts.has_velocities == True:
					out.write('{:{w}.{p}f}'.format(ag.velocities[i][0], w=15, p=8 ) )
					out.write('{:{w}.{p}f}'.format(ag.velocities[i][1], w=15, p=8 ) )
					out.write('{:{w}.{p}f}'.format(ag.velocities[i][2], w=15, p=8 ) )
				out.write('\n')
			out.close()
			break
		return True, filename

def Gmax_analysis(start, stop, step, Dir, outdir, cutoff, ca):
	cutoff = float(cutoff)

	gana = gmax_analysis()
	gana.get_steps(start, stop, step)
	gana.get_dir_names(Dir, outdir)
	elements = LICA.read_element_names(Dir, outdir)

	if(len(elements)==0):
		sys.stderr.write('[ERROR] Failed at tage 1 \n')
		return False
	gana.get_elemenrs(elements)

	status, file_name = gana.ananlysis()

	if(status == False):
		sys.stderr.write('[ERROR] Failed at tage 2 \n')
		return False

	poscar_file = LICA.read_info_file(Dir, outdir, file_name)

	status, nl_file = ATOMMAN.neighbour_list_create(poscar_file[0], outdir, start, cutoff)

	if(status == False):
		sys.stderr.write('[ERROR] Failed at tage 3 \n')
		return False

	status = ATOMMAN.neighbour_list_analysis(nl_file)

	if(status == False):
		sys.stderr.write('[ERROR] Failed at tage 4 \n')
		return False

	return True
