from ovito.io import import_file, export_file
from ovito.modifiers import CalculateDisplacementsModifier
import numpy
import lica.ana_std as ana

def XX():
    s1 = '#!/usr/local/bin/gnuplot -persist'
    s2 = 'set terminal postscript eps size 10cm,9cm  enhanced color font \" Times-New-Roman,18 \" linewidth 1'
    s3 = 'set output \"| epstopdf --filter --outfile=out.pdf\"'
    s4 = 'set key font ",18"'
    s5 = 'set xtics font "Times-Roman, 18" '
    s6 = 'set key samplen 3.0 spacing 1.5 font ",18"'
    s7 = 'set ytics offset 0.5,0'
    s8 = 'set xtics offset 0,0.3'
    s9 = 'set border -1 lw 2.0'
    s10 = 'set tics scale 3.0'
    s11 = 'set style line 1 lc rgb "blue"         lw 4 lt 10  pt 5  ps 2.0'
    s12 = 'set style line 2 lc rgb "red"          lw 4 lt 10  pt 9  ps 2.0'
    s13 = 'set style line 3 lc rgb "forest-green" lw 4 lt 10  pt 7  ps 2.0'
    s14 = 'set logscale xy'
    s15 = 'set xtics nomirror'
    s16 = 'set ytics nomirror'
    s17 = 'set ytics offset 0.5,0'
    s18 = 'set ylabel \'{/Time-New-Roman-Italic msd} (XX) \' offset 2.0,0,0 font ",15"'
    s19 = 'set xlabel \'{/Time-New-Roman-Italic time} (YY) \' offset 2.0,0,0 font ",15"'
    s20 = 'plot "outdir/msd_data.txt"  u 1:2 w l  ls 1 t \"\"'
    s= [s1, s2, s3, s4, s5, s6, s7, s8, s9, s10, s11, s12, s13, s14, s15, s16, s17, s18, s19, s20]
    return s

def write_plot():
    s = XX()
    out = open('plot.gnu', 'w')
    for i in range(len(s)):
        out.write(s[i])
        out.write('\n')
    out.close()


def read_file():
	if ana.IsFileExist('outdir/lammps_traj.dat') is False:
		sys.stderr.write('[ERROR] outdir/lammps_traj.dat absent \n')
		return False, []
	# Load input data and create a data pipeline.
	pipeline = import_file('outdir/lammps_traj.dat')
	return True, pipeline


# Calculate per-particle displacements with respect to initial simulation frame:
#pipeline.modifiers.append(CalculateDisplacementsModifier())

# Define the custom modifier function:
def calculate_msd(frame, data):

    # Access the per-particle displacement magnitudes computed by the 
    # CalculateDisplacementsModifier that precedes this user-defined modifier in the 
    # data pipeline:
    displacement_magnitudes = data.particles['Displacement Magnitude']

    # Compute MSD:
    msd = numpy.sum(displacement_magnitudes ** 2) / len(displacement_magnitudes)

    # Output MSD value as a global attribute: 
    data.attributes["MSD"] = msd

def process_calculate_msd():
	status, pipeline = read_file()
	if status is True:
		pipeline.modifiers.append(CalculateDisplacementsModifier())
		pipeline.modifiers.append(calculate_msd)
		# Export calculated MSD value to a text file and let OVITO's data pipeline do the rest:
		export_file(pipeline, "outdir/msd_data.txt",
		format = "txt/attr",
		columns = ["Timestep", "MSD"],
		multiple_frames = True)
		write_plot()
		return True
	else: return False
