import numpy
import sys
import os

if sys.version_info < (3,9):
	sys.exit('Require, Python >= 3.9')
try:
    from setuptools import setup, Extension
    use_setuptools = True
    print("setuptools is used.")
except ImportError:
    from distutils.core import setup, Extension
    use_setuptools = False
    print("distutils is used.")


cc = 'g++-9'
if 'CC' in os.environ:
	if 'g++-9' in os.environ['CC']:
		cc = 'g++-9'

include_dirs_numpy = []#[numpy.get_include()]
include_cpp_header = ['include']
include_gsl_header = []
include_dirs_lica = include_cpp_header + include_gsl_header + include_dirs_numpy
sources_cpp=['src/main_cluster_analysis.cpp',
			 'src/anastd.cpp', 
             'src/logger.cpp', 
             'src/gromax_info.cpp']

external_library_paths = []
extra_compile_args_lica = []
extra_link_args_lica = []

extension_lica = Extension('LICA',
					include_dirs = include_dirs_lica,
                    sources=sources_cpp,
                    define_macros = [],
                    undef_macros = [],
                    library_dirs = external_library_paths,
                    libraries = extra_link_args_lica)


ext_modules_lica = [extension_lica]

packages_lica = ['lica']

scripts_lica = ['scripts/lica']


if __name__ == "__main__":
	version_nums = [None, None, None]
	version_number = version_nums
	with open("lica/version.py") as f:
		for line in f:
			if "__version__" in line:
				for i, num in enumerate(line.split()[2].strip('\"').split('.')):
					version_nums[i] = int(num)
				break
	f.close()
	if None in version_nums:
		print("Failed to get version number in setup.py.")
		raise
	version_number = ".".join(["%d" % n for n in version_nums])
	if use_setuptools:
		setup(name = 'LICA',
				version = version_number,
				description = "This is LICA module.",
				author = "Pinku Nath",
				author_email = "pinkunath09@gmail.com",
				url = 'http://atztogo.github.io/phonopy/',
				zip_safe=False,
				packages = packages_lica,
				install_requires = [],
				provides = ['LICA'],
				scripts = scripts_lica,
				ext_modules = [extension_lica])
