#ifndef _MAIN_H_
#define _MAIN_H_
#include <main_header.h>
#endif


#ifndef _ANASTD_H_
#define _ANASTD_H_
#include <anastd_header.h>
#endif
#include <memory>


using namespace anastd;

gromax_info::gromax_info(Logger& l):_logger(l)
{
	star = std::string(25, '.');
	clear();
}
gromax_info::~gromax_info()
{
	//_logger.log.close();
	clear();
}
void gromax_info::clear()
{
	_atom_num.clear();
	_elements.clear();
	_elements_title.clear();
	_atom_types.clear();
	_atom_types_sorted.clear();
	_atom_indices.clear();
	_atom_indices_sorted.clear();
	anastd::clear_mat<double>(_lattice);
	anastd::clear_mat<double>(_positions);
}
//bool gromax_info::get_IO_name(char *forcefield_itp, char *outdir)
bool gromax_info::get_IO_name(char *forcefield_itp)
{
	//_dir = dir;
	//_outdir = outdir;

	_logger.log<<_LOG_MESSAGE_ <<"Directory name "<< _dir  <<" ..., "<<endl;
	//_logger.log<<_LOG_MESSAGE_ <<"Output files are directed to "<< _outdir  <<" ..., "<<endl;

	_forcefield_name = forcefield_itp; //_dir+"/forcefield.itp";

	if(!anastd::IsFileExist(_forcefield_name))
	{
        _logger.log << _LOG_ERROR_MESSAGE_ <<"filename_name : " << _forcefield_name << " not able open."<< endl;
		return false;
	}

	_logger.log<<_LOG_MESSAGE_ <<"Reading "<< _forcefield_name  <<" ..., "<<endl;

	if(!read_forcefield()) return false;

	return true;
}
bool gromax_info::get_IO_name(char *dir, char *outdir, char *info_file)
{
	//_dir = dir;
	_outdir = outdir;
	_info_file = info_file;

	//_logger.log<<_LOG_MESSAGE_ <<"Directory name "<< _dir  <<" ..., "<<endl;
	_logger.log<<_LOG_MESSAGE_ <<"Output files are directed to "<< _outdir  <<" ..., "<<endl;

	_forcefield_name = _dir+"/forcefield.itp";

	if(!anastd::IsFileExist(_forcefield_name))
	{
        _logger.log << _LOG_ERROR_MESSAGE_ <<"filename_name : " << _forcefield_name << " not able open."<< endl;
		return false;
	}

	_logger.log<<_LOG_MESSAGE_ <<"Reading "<< _forcefield_name  <<" ..., "<<endl;

	if(!read_forcefield()) return false;

	if(read_info())
	{
		if(organize_indices())
		{
			write_poscar();
		}
	}

	return true;
}
bool gromax_info::read_forcefield()
{
	ifstream infile (_forcefield_name.c_str());
	if(!infile.is_open()){
		_logger.log << _LOG_ERROR_MESSAGE_ <<"absent file: "+ _forcefield_name << endl;
        return false;
    }
    string line;
	bool read_status = false;
    while ( getline (infile,line) )
	{
        if (line == "") continue;
        if (line[0] == '#') continue;
        if (line[0] == '!') continue;
		if ((line.find(";name  at.num") != string::npos))
		{
			read_status = true;
		}else if ((line.find("nonbond_params") != string::npos))
		{
			read_status = false;
		}else if(read_status)
		{
			SVEC svec = anastd::split<string>(line);
			_elements.push_back(svec[0]);
		}
		
	}
    infile.clear();
    infile.close();

	if(_elements.size()==0)
	{
		_logger.log << _LOG_ERROR_MESSAGE_ <<" file format issue ...,"<< endl;
        return false;
	}

	_logger.log<<_LOG_MESSAGE_ << "Elements name"  <<" ..., "<<endl;

	for(uint i=0; i<_elements.size(); i++)
	{
		_logger.log << setw(5) << _elements[i];
	}
	_logger.log << endl;


	//element's name correction
	for(uint i=0; i<_elements.size(); i++)
	{
		string X = _elements[i];
		string S(1, X[0]);
		//string S = _elements[i][0];
		

		for(uint j=1; j<X.length(); j++)
		{
			char ch = char(tolower(X[j]));
			string s(1, ch);
			S = S + s;
		}
		_elements_title.push_back(S);
    }

	_logger.log<<_LOG_MESSAGE_ << "Elements name after corrections"  <<" ..., "<<endl;

	for(uint i=0; i<_elements_title.size(); i++)
	{
		_logger.log << setw(5) << _elements_title[i];
	}

	_logger.log << endl;

	return true;
}
bool gromax_info::read_info()
{
/*
	if(!anastd::IsFileExist(_info_file))
	{
        _logger.log << _LOG_ERROR_MESSAGE_ <<"filename_name : " << _info_file << " not able open."<< endl;
		return false;
	}

	_logger.log<<_LOG_MESSAGE_ << "Reading "<< _info_file  <<" ..., "<<endl;

	ifstream infile (_info_file);
	if(!infile.is_open()){
		_logger.log << _LOG_ERROR_MESSAGE_ <<"absent file: "+ _info_file << endl;
        return false;
    }
    string line;
    while ( getline (infile,line) )
	{
        if (line == "") continue;
        if (line[0] == '#') continue;
        if (line[0] == '!') continue;
		SVEC sv = split<string>(line);
		if(sv.size()==2)
		{
			DVEC v = split<double>(line);
			_time_step = v[0];
			_dt = v[1];
			
		}else if(sv.size()==3)
		{
			DVEC v = split<double>(line);
			_lattice.push_back(v);
		}else if(sv.size()==8)
		{
			_atom_types.push_back(sv[0]);
			_atom_indices.push_back(atoi(sv[1].c_str()));
			SVEC subvector = {sv.begin() + 2, sv.begin() + 5};
			DVEC pos(3, 0.0);
			pos[0] = atof(subvector[0].c_str());
			pos[1] = atof(subvector[1].c_str());
			pos[2] = atof(subvector[2].c_str());
			_positions.push_back(pos);
			subvector = {sv.begin() + 5, sv.end()};
			//cout<<subvector[0] << endl;
		}
	}
    infile.clear();
    infile.close();

	_logger.log << _LOG_MESSAGE_ <<"Time steps ...,"<< endl;
	_logger.log <<"time_step " << _time_step << " dt " << _dt << endl;
	_logger.log << _LOG_MESSAGE_ <<"Box size ...,"<< endl;

	for(uint i=0; i<3; i++)
	{
		for(uint j=0; j<3; j++)
		{
			_logger.log<< setw(15) << _lattice[i][j];
		}
		_logger.log<< endl;
	}
*/
	return true;
}
bool gromax_info::organize_indices()
{
/*
	if(_elements.size()==0) return false;
	else if(_atom_types.size()==0) return false;
	else if(_atom_indices.size()==0) return false;


	_atom_indices_sorted.resize(_atom_indices.size(), -10);
	//_atom_types_sorted.resize(_atom_types.size(), "");

	_atom_num.resize(_elements.size(), 0);

	uint cnt=0;
	for(uint i=0; i<_elements.size(); i++)
	{
		for(uint j=0; j<_atom_types.size(); j++)
		{
			if(_elements[i]==_atom_types[j])
			{
				_atom_num[i]+=1;
				_atom_indices_sorted[cnt] = j;
				//_atom_types_sorted[cnt] = _elements[i];
				cnt++;
			}
		}
	}
	uint TA = 0;
	for(uint i=0; i<_atom_num.size(); i++)
	{
		TA+=_atom_num[i];
	}

	_logger.log << _LOG_MESSAGE_ <<"Atom types ...,"<< endl;

	_logger.log << star<< endl;

	for(uint i=0; i<_elements.size(); i++)
	{
		_logger.log<< setw(5) << _elements[i];
	}
	_logger.log<< endl;
	_logger.log << star<< endl;

	for(uint i=0; i<_atom_num.size(); i++)
	{
		_logger.log<< setw(5) << _atom_num[i];
	}
	_logger.log<< endl;

	_logger.log<<_LOG_MESSAGE_  << "Total Atom " << TA << endl;

	_logger.log<<_LOG_MESSAGE_  << "sorted_indices " <<" Types" << endl;

	_logger.log << star<< endl;
	for(uint i=0; i<_atom_indices_sorted.size(); i++)
	{
		_logger.log<< setw(5) << _atom_indices_sorted[i] << setw(10) << _atom_types[_atom_indices_sorted[i]] << endl;
	}
*/
	return true;
}
bool gromax_info::write_poscar()
{
/*
	_poscar_file = _outdir+"/poscar_"+to_string((int)_time_step)+".vasp";
	_logger.log << _LOG_MESSAGE_ << "Writing poscar " << _poscar_file << endl;

	ofstream outfile;
    outfile.open (_poscar_file);

    if(!outfile.is_open())
    {
		_logger.log << _LOG_ERROR_MESSAGE_ <<"absent file: "+ _poscar_file << endl;
        return false;
    }
    outfile << std::setiosflags(std::ios::fixed | std::ios::showpoint | std::ios::right);
    outfile << setprecision(15);

    outfile <<"POSCAR-LICA"<< endl;
    outfile <<"1.0000"<<endl;

	for(uint i=0; i<3; i++)
	{
		for(uint j=0; j<3; j++)
		{
			outfile << setw(25) << _lattice[i][j];
		}
		 outfile << endl;
	}

    for(uint i=0; i<_elements.size(); i++)
    {
        outfile << setw(8) << _elements_title[i];
    }
    outfile << endl;
    for(uint i=0; i<_atom_num.size(); i++)
    {
        outfile << setw(8) << _atom_num[i];
    }
    outfile << endl;

	outfile<<"Cartesian" << endl;
	for(uint i=0; i<_atom_indices_sorted.size(); i++)
	{
		outfile<< setw(25) << _positions[_atom_indices_sorted[i]][0];  
		outfile<< setw(25) << _positions[_atom_indices_sorted[i]][1];  
		outfile<< setw(25) << _positions[_atom_indices_sorted[i]][2];
		string S=_atom_types[_atom_indices_sorted[i]];

		for(uint j=0; j<_elements.size(); j++)
		{
			if(_elements[j] == S)
			{
				S.replace(S.begin(),S.end(),_elements_title[j].begin(),_elements_title[j].end());
			}
		}
		outfile<< setw(5) << S;  
		outfile << endl;
	}

	outfile.clear();
	outfile.close();
*/
	return true;
}
